# -*- coding: utf-8 -*-
import json
import os
import random
import shutil
import sys
import time

from datetime import datetime

# Azure AI Vision用ライブラリをインポートする。
from azure.cognitiveservices.vision.computervision import ComputerVisionClient
from azure.cognitiveservices.vision.computervision.models import OperationStatusCodes
from msrest.authentication import CognitiveServicesCredentials

# 日付をYYYY/MM/DD HH:MM:SS形式で取得する。
def get_time():
    now = datetime.now()
    str = now.strftime("%Y/%m/%d %H:%M:%S")
    return str

# 日付をYYYYMMDD形式で取得する。
def get_date_without_slash():
    now = datetime.now()
    str = now.strftime("%Y%m%d")
    return str

# ログ出力を行う。
def log(mes):
    global logfile
    with open(logfile, mode='a', encoding='utf-8') as l:
        l.write(f'{get_time()} {mes}\n')

# LICを取得する。
def get_lic(s, a):
    len1 = len(s)
    random.seed(a)
    code6 = []
    for i in range(len1):
        code6.append(str2code6[s[i]])
    code8 = []
    bit = 0
    k = 0
    for i in range(len1):
        for j in range(6):
            if k % 8 == 0:
                if (i != 0 or j != 0):
                    code8.append(bit)
                if (code6[i] & (0x20 >> j)) != 0:
                    bit = 0x80 >> (k % 8)
                else:
                    bit = 0
            else:
                if (code6[i] & (0x20 >> j)) != 0:
                    bit |= (0x80 >> (k % 8))
            k += 1
    if(k % 8 == 0):
        code8.append(bit)
    len8 = len(code8)
                         
    code8decode = []
    for i in range(len8):
        code8decode.append(code8[i] ^ random.randint(0,255))
    return  bytes(code8decode).decode('utf-8')

# 指定されたディレクトリ内の画像ファイル名を取得する。
def get_image_files(directory):
    image_extensions = ('.jpg', '.jpeg', '.png', '.tif')
    try:
        files = [file for file in os.listdir(directory) if file.lower().endswith(image_extensions)]
    except Exception as e:
        log(e)
        files = []
    return files

# 指定された画像ファイルを送信し、結果を受け取ってテキストファイルに保存する。
def get_ocr_result(client, image_path, image_file, folder_sumi, tate=False):
    # ローカルの画像ファイルを読み込む。
    image_path += '\\' + image_file
    log(f'{image_path}を処理します。')
    with open(image_path, mode='rb') as g:
        # OCRリクエストを送信する。
        try:
            response = client.read_in_stream(g, language='ja', raw=True)
        except Exception as e:
            log(f'画像ファイル読み込み時にエラーが発生しました。{e}')
            return

    try:
        # レスポンスから操作場所（末尾にIDを持つURL）を取得する。
        read_operation_location = response.headers['Operation-Location']
        # URLからIDを取得する。
        operation_id = read_operation_location.split('/')[-1]

        # GET APIを呼び出し、結果を取得するのを待つ。
        while True:
            read_result = client.get_read_result(operation_id)
            if read_result.status not in ['notStarted', 'running']:
                break
            time.sleep(1)
    except Exception as e:
        log(f'レスポンス取得時にエラーが発生しました。{e}')
        return

    # 検出したテキストをファイル出力する。
    if read_result.status == OperationStatusCodes.succeeded:
        output_file = os.path.splitext(image_path)[0] + '.txt'
        with open(output_file, mode='w', encoding='utf-8') as t:
            for text_result in read_result.analyze_result.read_results:
                if tate:
                    # 縦原稿の場合、行の順序を逆にして出力する。
                    for line in reversed(text_result.lines):
                        t.write(line.text + '\n')
                else:
                    # 横原稿の場合、行の順序はそのまま出力する。
                    for line in text_result.lines:
                        t.write(line.text + '\n')

    # 処理が終わったファイルを済フォルダへ移動させる。
    try:
        shutil.move(image_path, folder_sumi)
    except Exception as e:
        log(f'処理済み画像ファイルの移動でエラーが発生しました。{e}')

    return


# 設定ファイルを読み込む。
inifile = 'C:\\OCR\\ocr.json'
if not os.path.exists(inifile):
    print(f'{inifile}がありません。')
    exit()

with open(inifile, mode='r', encoding='utf-8') as j:
    config = json.load(j)

# 設定ファイルから設定を取得する。
#  APIキー
#  エンドポイント
#  縦型のファイルを置くフォルダ
#  横型のファイルを置くフォルダ
#  ログファイル
#  LIC
try:
    api_key = config['api_key']
except Exception as e:
    print('ocr.jsonファイルにapi_keyエントリがありません。')
    exit()

try:
    endpoint = config['endpoint']
except Exception as e:
    print('ocr.jsonファイルにendpointエントリがありません。')
    exit()

try:
    image_path_tate = os.path.normpath(config['image_path_tate'])
except Exception as e:
    print('ocr.jsonファイルにimage_path_tateエントリがありません。')
    exit()

try:
    image_path_yoko = os.path.normpath(config['image_path_yoko'])
except Exception as e:
    print('ocr.jsonファイルにimage_path_yokoエントリがありません。')
    exit()

try:
    global logfile
    logfile = config['logfile']
except Exception as e:
    print('ocr.jsonファイルにlogfileエントリがありません。')
    exit()

try:
    lic = config['lic']
except Exception as e:
    print('ocr.jsonファイルにlicエントリがありません。')
    exit()

# エラーチェック
if api_key == '':
    print('ocr.jsonファイルにAPIキーが設定されていません。')
    exit()
if endpoint == '':
    print('ocr.jsonファイルにエンドポイントが設定されていません。')
    exit()
if image_path_tate == '.':
    print('ocr.jsonファイルに縦原稿フォルダが設定されていません。')
    exit()
if image_path_yoko == '.':
    print('ocr.jsonファイルに横原稿フォルダが設定されていません。')
    exit()
if logfile == '':
    print('ocr.jsonファイルにログファイルが設定されていません。')
    exit()
if lic == '':
    print('ocr.jsonファイルにライセンス情報が設定されていません。')
    exit()

# ライセンス情報をチェックする。
code62str = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
           'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', '-', '=',
           'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
           'U', 'V', 'W', 'X', 'Y', 'Z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',]
str2code6 = {}
for i in range(len(code62str)):
    str2code6[code62str[i]]=i

lic_file = int(get_lic(lic, 64))
lic_now = int(get_date_without_slash())
if lic_now > lic_file:
    log(f'ライセンスがありません。開発元にライセンス送付を依頼してください。')
    exit()

# 処理済みフォルダを作成する。
# exist_ok=Trueだと、既にあってもエラーにならない。
try:
    sumi_tate = image_path_tate + '\\済'
    os.makedirs(sumi_tate, exist_ok=True)
except Exception as e:
    log(f'{sumi_tate}フォルダ作成時にエラーが発生しました。{e}')
    exit()
try:
    sumi_yoko = image_path_yoko + '\\済'
    os.makedirs(sumi_yoko, exist_ok=True)
except Exception as e:
    log(f'{sumi_yoko}フォルダ作成時にエラーが発生しました。{e}')
    exit()

# 置かれている画像ファイルを取得する。
image_files_tate = get_image_files(image_path_tate)
image_files_yoko = get_image_files(image_path_yoko)

# Computer Visionクライアントを作成する。
# APIキーとエンドポイントが間違っていても、エラーにはならない。
# そのため例外処理は不要。
client = ComputerVisionClient(endpoint, CognitiveServicesCredentials(api_key))

# 縦原稿の処理を行う。
for f in image_files_tate:
    print(f'-----{f}-----')
    get_ocr_result(client, image_path_tate, f, sumi_tate, tate=True)
    time.sleep(3)

# 横原稿の処理を行う。
for f in image_files_yoko:
    print(f'-----{f}-----')
    get_ocr_result(client, image_path_yoko, f, sumi_yoko)
    time.sleep(3)
